/*
 *		Jiri Skala		A01216
 *
 *		KIV / GSVD		2004/05
 *
 *		INTERNAL TESTING MODULE
 */
using System;
using System.Xml;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;

namespace Zcu.Gsvd.DisplaceMap.Testing
{
	/// <summary>
	/// <p>Source of regular 2D grid. Supports cell color attributes
	/// and point scalar attributes. This is internal module
	/// for testing purposes only.</p>
	/// </summary>
	[ModuleInfo("Ji Skla", "Source of regular 2D grid. For testing purposes only.")]
	[PortInfo("Output", "Regular 2D grid.")]
	public class RegGrid2DSource : Zcu.Mve.Core.Module
	{
		/// <summary>
		/// Grid width (counted in cells).
		/// </summary>
		private int width;

		/// <summary>
		/// Grid height (counted in cells).
		/// </summary>
		private int height;

		/// <summary>
		/// Determines whether created grid should have cell color attributes.
		/// </summary>
		private bool assignCellColor;
		/// <summary>
		/// Determines whether created grid should have cell scalar attributes.
		/// </summary>
		private bool assignCellScalar;
		/// <summary>
		/// Determines whether created grid should have point color attributes.
		/// </summary>
		private bool assignPointColor;
		/// <summary>
		/// Determines whether created grid should have point scalar attributes.
		/// </summary>
		private bool assignPointScalar;

		/// <summary>
		/// Cell color attribute name.
		/// </summary>
		private string cellColorName;
		/// <summary>
		/// Cell scalar attribute name.
		/// </summary>
		private string cellScalarName;
		/// <summary>
		/// Point color attribute name.
		/// </summary>
		private string pointColorName;
		/// <summary>
		/// Point scalar attribute name.
		/// </summary>
		private string pointScalarName;

		/// <summary>
		/// Random number generator.
		/// </summary>
		private Random rnd;

		/// <summary>
		/// Constructor. Creates output port.
		/// </summary>
		public RegGrid2DSource()
		{
			// initialize
			rnd = new Random();
			width = 5;
			height = 3;
			assignCellColor = true;
			cellColorName = typeof(ColorRGBA).Name;
			assignCellScalar = false;
			cellScalarName = typeof(Scalar).Name;
			assignPointColor = false;
			pointColorName = typeof(ColorRGBA).Name;
			assignPointScalar = true;
			pointScalarName = typeof(Scalar).Name;

			// create output port
			AddOutPort("Output", typeof(RegGrid2D));
		}

		#region Properties

		/// <summary>
		/// Gets or sets grid width (counted in cells).
		/// </summary>
		internal int Width
		{
			get { return width; }	set { width = value; }
		}

		/// <summary>
		/// Gets or sets grid height (counted in cells).
		/// </summary>
		internal int Height
		{
			get { return height; }	set { height = value; }
		}

		/// <summary>
		/// Gets cell color attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="assign">True if the attribute should be assigned.</param>
		/// <returns>Returns cell color attribute name.</returns>
		internal string GetCellColorName(out bool assign)
		{
			assign = assignCellColor;
			return cellColorName;
		}
		/// <summary>
		/// Sets cell color attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="name">Cell color attribute name.</param>
		/// <param name="assign">True if the attribute should be assigned.</param>
		internal void SetCellColorName(string name, bool assign)
		{
			cellColorName = name;
			assignCellColor = assign;
		}

		/// <summary>
		/// Gets cell scalar attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="assign">True if the attribute should be assigned.</param>
		/// <returns>Returns cell scalar attribute name.</returns>
		internal string GetCellScalarName(out bool assign)
		{
			assign = assignCellScalar;
			return cellScalarName;
		}
		/// <summary>
		/// Sets cell scalar attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="name">Cell scalar attribute name.</param>
		/// <param name="assign">True if the attribute should be assigned.</param>
		internal void SetCellScalarName(string name, bool assign)
		{
			cellScalarName = name;
			assignCellScalar = assign;
		}

		/// <summary>
		/// Gets point color attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="assign">Determines whether the attribute should be assigned.</param>
		/// <returns>Returns color scalar attribute name.</returns>
		internal string GetPointColorName(out bool assign)
		{
			assign = assignPointColor;
			return pointColorName;
		}
		/// <summary>
		/// Sets point color attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="name">Point color attribute name.</param>
		/// <param name="assign">True if the attribute should be assigned.</param>
		internal void SetPointColorName(string name, bool assign)
		{
			pointColorName = name;
			assignPointColor = assign;
		}

		/// <summary>
		/// Gets point scalar attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="assign">Determines whether the attribute should be assigned.</param>
		/// <returns>Returns point scalar attribute name.</returns>
		internal string GetPointScalarName(out bool assign)
		{
			assign = assignPointScalar;
			return pointScalarName;
		}
		/// <summary>
		/// Sets point scalar attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="name">Point scalar attribute name.</param>
		/// <param name="assign">True if the attribute should be assigned.</param>
		internal void SetPointScalarName(string name, bool assign)
		{
			pointScalarName = name;
			assignPointScalar = assign;
		}

		#endregion

		/// <summary>
		/// Creates a deep copy of this instance.
		/// </summary>
		/// <returns>Returns a deep copy of this instance.</returns>
		public override Module DeepCopy()
		{
			RegGrid2DSource copy = (RegGrid2DSource)base.DeepCopy();

			copy.Width = this.width;
			copy.Height = this.height;
			copy.SetCellColorName(this.cellColorName, this.assignCellColor);
			copy.SetCellScalarName(this.cellScalarName, this.assignCellScalar);
			copy.SetPointColorName(this.pointColorName, this.assignPointColor);
			copy.SetPointScalarName(this.pointScalarName, this.assignPointScalar);

			return copy;
		}

		/// <summary>
		/// Saves current module settings.
		/// </summary>
		/// <param name="config">XML element where to save.</param>
		public override void WriteConfig(System.Xml.XmlElement config)
		{
			// save grid dimensions
			XmlElement size = config.OwnerDocument.CreateElement("size");
			size.SetAttribute("width", width.ToString());
			size.SetAttribute("height", height.ToString());
			config.AppendChild(size);

			// save cell color attribute setting
			XmlElement cell = config.OwnerDocument.CreateElement("cellAttribute");
			cell.SetAttribute("assign", assignCellColor.ToString());
			cell.InnerText = cellColorName;
			config.AppendChild(cell);

			// save point scalar attribute setting
			XmlElement point = config.OwnerDocument.CreateElement("pointAttribute");
			point.SetAttribute("assign", assignPointScalar.ToString());
			point.InnerText = pointScalarName;
			config.AppendChild(point);
		}

		/// <summary>
		/// Loads module settings.
		/// </summary>
		/// <param name="config">XML element from where to load.</param>
		public override void ReadConfig(System.Xml.XmlElement config)
		{
			// load module setup
			foreach (XmlElement element in config.ChildNodes)
				switch (element.Name)
				{
					case "size":
						width = int.Parse(element.GetAttribute("width"));
						height = int.Parse(element.GetAttribute("height"));
						break;

					case "cellAttribute":
						assignCellColor = bool.Parse(element.GetAttribute("assign"));
						cellColorName = element.InnerText.Trim();
						break;

					case "pointAttribute":
						assignPointScalar = bool.Parse(element.GetAttribute("assign"));
						pointScalarName = element.InnerText.Trim();
						break;
				}
		}

		/// <summary>
		/// <p><i>RegGrid2DSource</i> module setup dialog.</p>
		/// </summary>
		/// <remarks>Creates module setup dialog.</remarks>
		/// <returns>Returns module setup dialog reference.</returns>
		public override ModuleSetup InvokeSetup()
		{
			return new RegGrid2DSourceSetup(this);
		}

		/// <summary>
		/// Creates sample regular 2D grid and assigns cell / point attributes if desired.
		/// </summary>
		public override void Execute()
		{
			// create new regular 2D grid
			RegGrid2D grid = new RegGrid2D(width, height);

			if (assignCellColor)
			{
				// prepare array for attributes
				UniformDataArray2D cellAttrs
					= new UniformDataArray2D(typeof(ColorRGBA), width, height);

				// create color attributes
				for (int j = 0; j < height; j++)
					for (int i = 0; i < width; i++)
					{
						// create some random color
						int r = rnd.Next(256);
						int g = rnd.Next(256);
						int b = rnd.Next(256);
						// store color into attribute array
						cellAttrs[i, j] = new ColorRGBA(r, g, b);
					}

				// assign cell attributes
				grid.AddCellAttrs(cellColorName, cellAttrs);
			}

			if (assignCellScalar)
			{
				// prepare array for attributes
				UniformDataArray2D cellAttrs
					= new UniformDataArray2D(typeof(Scalar), width, height);

				// create scalar attributes
				for (int j = 0; j < height; j++)
					for (int i = 0; i < width; i++)
						// create some random scalar
						cellAttrs[i, j] = new Scalar(rnd.NextDouble());

				// assign cell attributes
				grid.AddCellAttrs(cellScalarName, cellAttrs);
			}

			if (assignPointColor)
			{
				// prepare array for attributes
				UniformDataArray2D pointAttrs
					= new UniformDataArray2D(typeof(ColorRGBA), width+1, height+1);

				// create color attributes
				for (int j = 0; j < height+1; j++)
					for (int i = 0; i < width+1; i++)
					{
						// create some random color
						int r = rnd.Next(256);
						int g = rnd.Next(256);
						int b = rnd.Next(256);
						// store color into attribute array
						pointAttrs[i, j] = new ColorRGBA(r, g, b);
					}

				// assign cell attributes
				grid.AddPointAttrs(pointColorName, pointAttrs);
			}

			if (assignPointScalar)
			{
				// prepare array for attributes
				UniformDataArray2D pointAttrs
					= new UniformDataArray2D(typeof(Scalar), width+1, height+1);

				// create scalar attributes
				for (int j = 0; j < height+1; j++)
					for (int i = 0; i < width+1; i++)
						// create some random scalar
						pointAttrs[i, j] = new Scalar(rnd.NextDouble());

				// assign point attributes
				grid.AddPointAttrs(pointScalarName, pointAttrs);
			}

			// publish output
			SetOutput("Output", grid);
		}
	}
}