/*
 *		Jiri Skala		A01216
 *
 *		KIV / GSVD		2004/05
 *
 *		INTERNAL TESTING MODULE
 */
using System;
using System.Xml;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;

namespace Zcu.Gsvd.DisplaceMap.Testing
{
	/// <summary>
	/// <p>Adds point color and point scalar attributes to triangle mesh.
	/// This is internal module for testing purposes only.</p>
	/// </summary>
	[ModuleInfo("Ji Skla", "Adds point color and point scalar attributes "
		+ "to triangle mesh. For testing purposes only.")]
	[PortInfo("Input", "Input triangle mesh.")]
	[PortInfo("Output", "Output triangle mesh with attributes added.")]
	public class TriMeshAttrAdder : Zcu.Mve.Core.Module
	{
		/// <summary>
		/// Determines whether point color attributes should be added to triangle mesh.
		/// </summary>
		private bool assignPointColor;

		/// <summary>
		/// Determines whether point scalar attributes should be added to triangle mesh.
		/// </summary>
		private bool assignPointScalar;

		/// <summary>
		/// Point color attribute name.
		/// </summary>
		private string pointColorName;

		/// <summary>
		/// Point scalar attribute name.
		/// </summary>
		private string pointScalarName;

		/// <summary>
		/// Determines whether input triangle mesh should be converted from 3D to 2D.
		/// </summary>
		private bool makeMeshPlanar;

		/// <summary>
		/// Random number generator.
		/// </summary>
		private Random rnd;

		/// <summary>
		/// Constructor. Creates module ports.
		/// </summary>
		public TriMeshAttrAdder()
		{
			// initialize
			assignPointColor = true;
			assignPointScalar = true;
			pointColorName = typeof(ColorRGBA).Name;
			pointScalarName = typeof(Scalar).Name;
			makeMeshPlanar = false;
			rnd = new Random();

			// create ports
			AddInPort("Input", typeof(TriangleMesh));
			AddOutPort("Output", typeof(TriangleMesh));
		}

		/// <summary>
		/// Gets point color attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="assign">True if the attribute should be assigned.</param>
		/// <returns>Returns point color attribute name.</returns>
		internal string GetPointColorName(out bool assign)
		{
			assign = assignPointColor;
			return pointColorName;
		}

		/// <summary>
		/// Sets point color attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="name">Point color attribute name.</param>
		/// <param name="assign">True if the attribute should be assigned.</param>
		internal void SetPointColorName(string name, bool assign)
		{
			assignPointColor = assign;
			pointColorName = name;
		}

		/// <summary>
		/// Gets point scalar attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="assign">True if the attribute should be assigned.</param>
		/// <returns>Returns point scalar attribute name.</returns>
		internal string GetPointScalarName(out bool assign)
		{
			assign = assignPointScalar;
			return pointScalarName;
		}

		/// <summary>
		/// Sets point scalar attribute name and whether the attribute should be assigned.
		/// </summary>
		/// <param name="name">Point scalar attribute name.</param>
		/// <param name="assign">True if the attribute should be assigned.</param>
		internal void SetPointScalarName(string name, bool assign)
		{
			assignPointScalar = assign;
			pointScalarName = name;
		}

		/// <summary>
		/// Gets or sets whether mesh should be converted to 2D.
		/// </summary>
		internal bool MakePlanar
		{
			get { return makeMeshPlanar; }
			set { makeMeshPlanar = value; }
		}

		/// <summary>
		/// Creates a deep copy of this instance.
		/// </summary>
		/// <returns>Returns a deep copy of this instance.</returns>
		public override Module DeepCopy()
		{
			TriMeshAttrAdder copy = (TriMeshAttrAdder)base.DeepCopy();

			copy.SetPointColorName(pointColorName, assignPointColor);
			copy.SetPointScalarName(pointScalarName, assignPointScalar);

			return copy;
		}

		/// <summary>
		/// Saves current module settings.
		/// </summary>
		/// <param name="config">XML element where to save.</param>
		public override void WriteConfig(System.Xml.XmlElement config)
		{
			// save point color attribute setting
			XmlElement color = config.OwnerDocument.CreateElement("pointColor");
			color.SetAttribute("assign", assignPointColor.ToString());
			color.InnerText = pointColorName;
			config.AppendChild(color);

			// save point scalar attribute setting
			XmlElement scalar = config.OwnerDocument.CreateElement("pointScalar");
			scalar.SetAttribute("assign", assignPointScalar.ToString());
			scalar.InnerText = pointScalarName;
			config.AppendChild(scalar);

			// save 3D to 2D conversion setting
			XmlElement planar = config.OwnerDocument.CreateElement("makePlanar");
			planar.InnerText = makeMeshPlanar.ToString();
			config.AppendChild(planar);
		}

		/// <summary>
		/// Loads module settings.
		/// </summary>
		/// <param name="config">XML element from where to load.</param>
		public override void ReadConfig(System.Xml.XmlElement config)
		{
			// load module setup
			foreach (XmlElement element in config.ChildNodes)
				switch (element.Name)
				{
					case "pointColor":
						assignPointColor = bool.Parse(element.GetAttribute("assign"));
						pointColorName = element.InnerText.Trim();
						break;

					case "pointScalar":
						assignPointScalar = bool.Parse(element.GetAttribute("assign"));
						pointScalarName = element.InnerText.Trim();
						break;

					case "makePlanar":
						makeMeshPlanar = bool.Parse(element.InnerText.Trim());
						break;
				}
		}

		/// <summary>
		/// <p><i>TriMeshAttrAdder</i> module setup dialog.</p>
		/// </summary>
		/// <remarks>Creates module setup dialog.</remarks>
		/// <returns>Returns module setup dialog reference.</returns>
		public override ModuleSetup InvokeSetup()
		{
			return new TriMeshAttrAdderSetup(this);
		}

		/// <summary>
		/// Adds attributes to triangle mesh points.
		/// </summary>
		public override void Execute()
		{
			// get module input
			TriangleMesh input = (TriangleMesh)GetInput("Input");

			// create shallow copy of original triangle mesh
			// invalid cast: TriangleMesh mesh = (TriangleMesh)input.ShallowCopy();
			TriangleMesh mesh = new TriangleMesh();
			string triName = typeof(Triangle).Name;
			// copy reference to points
			mesh.Points = input.Points;
			// copy point attributes
			foreach (string attrName in input.GetPointAttrsIds())
				mesh.AddPointAttrs(attrName, input.GetPointAttrs(attrName));
			// copy reference to cells
			mesh.AddCells(triName, input.GetCells(triName));
			// copy cell attributes
			foreach (string attrName in input.GetCellAttrsIds(triName))
				mesh.AddCellAttrs(triName, attrName, input.GetCellAttrs(triName, attrName));

			// check whether mesh should be converted to 2D
			if (makeMeshPlanar)
			{
				// prepare array for points
				UniformDataArray points
					= new UniformDataArray(typeof(Point2D), input.Points.Count);

				// copy points and discard Z coordinates
				for (int i = 0; i < points.Count; i++)
				{
					// get 3D point and create 2D point
					// invalid cast: points[i] = (Point2D)input.Points[i];
					// get 3D point
					Point3D p = (Point3D)input.Points[i];
					// create 2D point
					points[i] = new Point2D(p.X, p.Y);
				}

				// overwrite points with new ones
				mesh.Points = points;
			}

			if (assignPointColor)
			{
				// prepare array for point color attributes
				UniformDataArray colors
					= new UniformDataArray(typeof(ColorRGBA), mesh.Points.Count);

				// create color attributes
				for (int i = 0; i < colors.Count; i++)
				{
					// create some random color
					int r = rnd.Next(256);
					int g = rnd.Next(256);
					int b = rnd.Next(256);
					// store color into attribute array
					colors[i] = new ColorRGBA(r, g, b);
				}

				// assign cell attributes
				mesh.AddPointAttrs(pointColorName, colors);
			}

			if (assignPointScalar)
			{
				// prepare array for point scalar attributes
				UniformDataArray scalars
					= new UniformDataArray(typeof(Scalar), mesh.Points.Count);

				// create scalar attributes
				for (int i = 0; i < scalars.Count; i++)
					// create some random scalar
					scalars[i] = new Scalar(rnd.NextDouble());

				// assign cell attributes
				mesh.AddPointAttrs(pointScalarName, scalars);
			}

			// publish output
			SetOutput("Output", mesh);
		}
	}
}